/*
    Copyright (C) 2002  bjk <bjk@arbornet.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <sys/types.h>
#include <sys/stat.h>

#ifdef __linux__
#include <getopt.h>
#endif

#ifndef LINE_MAX
#define LINE_MAX	2048
#endif

#define PROGNAME	"bubblegum"
#define VERSION 	"1.8"
#define COPYRIGHT 	"Copyright 2002 bjk <bjk@arbornet.org>"
#define LOGFILE		"bubblegum.log"
#define CMDRUNS		1
#define LOGLEVEL	2
#define INTERVAL	1
#define SYSLOG_FACILITY	LOG_DAEMON
#define SYSLOG_PRIORITY	LOG_NOTICE
#define TIMEFORMAT 	"%b %d %H:%M:%S"
#define MAX_TIME_LEN 	16
#define NONE_STR	"-"
#define ATIME_STR	"ATIME"
#define MTIME_STR	"MTIME"
#define CTIME_STR	"CTIME"
#define ERROR_STR	"ERROR"
#define IDEV_STR	"IDEV"
#define INODE_STR	"INODE"
#define SIZE_STR	"SIZE"
#define UID_STR		"UID"
#define GID_STR		"GID"
#define HLINKS_STR	"HLINK"
#define PERMS_STR	"PERM"
#define DIR_STR		"DIRECTORY"
#define CHR_STR		"CHARACTER"
#define BLK_STR		"BLOCK"
#define REG_STR		"REGULAR"
#define FIFO_STR	"FIFO"
#define LNK_STR		"LINK"
#define SOCK_STR	"SOCKET"

#define SET_BIT(var, bit)	(var |= (1 << bit))
#define RESET_BIT(var, bit)	(var &= ~(1 << bit))
#define TEST_BIT(var, bit)	(var & (1 << bit))
#define TOGGLE_BIT(var, bit)	(var ^= (1 << bit))

#define STAT(file, sstat) ((symlinks) ? lstat(file, sstat) : stat(file, sstat))

enum { ACCESS = 1, MODIFY, CHANGE, ERROR, MAXCHGBITS };
enum { IDEV = 1, INODE, SIZE, UID, GID, HLINKS, PERMS, MAXTYPEBITS };
enum { NORMAL, RESET, RESETALL, RELOAD, QUIT };

struct filelist {
    char *filename;
    struct stat laststat;
    unsigned staterrno;
    unsigned changed;
    unsigned chgtype;
    unsigned cmdruns;
    struct filelist *next;
};

unsigned check;
unsigned usesyslog, total, symlinks, runlevel;
int niceness, loglevel;
long interval;
char *logfile, *wd, *filenamelist;
char command[FILENAME_MAX];
pid_t pid, cpid;

struct filelist *filecmp(const char *filename, struct filelist *list);
struct filelist *loadfile(const char *filename, struct filelist *oldlist);
int duplicate(const char *filename, struct filelist *list);
int is_file(const char *filename);
void logit(const char *format, ...);
void *Calloc(size_t number, size_t size);
void *Malloc(size_t size);
int pathlength(const char *wd, const char *filename);
char *fullpath(const char *wd, char *filename, char dest[], size_t size);
