/*
    Copyright (C) 2002  bjk <bjk@arbornet.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "bubblegum.h"

struct filelist *
filecmp (const char *filename, struct filelist *list)
{
  struct filelist *cur, *next;

  for (cur = list; cur; cur = next)
    {
      next = cur->next;

      if (!cur->filename)
	break;

      if (strcmp (filename, cur->filename) == 0)
	return cur;
    }

  return NULL;
}

int
duplicate (const char *filename, struct filelist *list)
{
  struct filelist *cur, *next;

  for (cur = list; cur; cur = next)
    {
      next = cur->next;

      if (!cur->filename)
	break;

      if (strcmp (filename, cur->filename) == 0)
	return 1;
    }

  return 0;
}

struct filelist *
loadfile (const char *filename, struct filelist *oldlist)
{
  FILE *fp;
  char *cp, *buf;
  int err, count = 0;
  struct filelist *head, *flist;

  if (oldlist && (err = is_file (filenamelist)) != 0)
    {
      if (err < 0)
	logit ("%s: %s", filenamelist, strerror (errno));
      else
	logit ("%s: not a file", filenamelist);

      return NULL;
    }

  if ((fp = fopen (filename, "r")) == NULL)
    return NULL;

  head = (struct filelist *) Calloc (1, sizeof (struct filelist));
  flist = head;
  buf = Calloc (1, LINE_MAX);

  while ((cp = fgets (buf, LINE_MAX, fp)) != NULL)
    {
      struct stat mystat;
      struct filelist *cur;
      char *tmp;
      int bufsize = 0;

      buf[strlen (buf) - 1] = '\0';
      bufsize = pathlength (wd, buf) + 1;

      tmp = Malloc (bufsize);
      cp = fullpath (wd, cp, tmp, bufsize);

      if (duplicate (cp, head))
	{
	  flist->next = NULL;

	  if (oldlist)
	    logit ("%s: duplicate file", cp);
	  else
	    fprintf (stderr, "%s: duplicate file\n", cp);

	  free (tmp);
	  continue;
	}

      /*
       * reload list keeping old stat and bit data 
       */
      if (oldlist)
	{
	  if ((cur = filecmp (cp, oldlist)) != NULL)
	    {
	      bcopy (cur, flist, sizeof (struct filelist));
	      flist->filename = strdup (cur->filename);
	      flist->next = Calloc (1, sizeof (struct filelist));
	      flist = flist->next;

	      free (tmp);
	      count++;
	      continue;
	    }
	}

      if (STAT (cp, &mystat) == -1)
	{
	  if (oldlist)
	    logit ("%s: %s", cp, strerror (errno));
	  else
	    perror (cp);

	  free (tmp);
	  continue;
	}

      flist->filename = Calloc (1, bufsize);
      strncpy (flist->filename, cp, bufsize);

      flist->laststat = mystat;
      flist->next = Calloc (1, sizeof (struct filelist));
      flist = flist->next;

      count++;
      free (tmp);
    }

  flist->next = NULL;

  free (buf);
  fclose (fp);

  if (!count && oldlist)
    {
      logit ("%s: no files", filenamelist);
      free (head);
      return NULL;
    }

  total = count;
  return head;
}
